# Proj300 —— 为 Electron 构建工具在 RISC-V 平台上提供支持

## 项目展示

目前是启用在 docker 中的 riscv64 环境下的 arch 系统，使用 electron-builder 工具成功进行打包构建，成功生成的 tar.xz 文件。

![alt text](assets/README/image-9.png)
![alt text](assets/README/image-10.png)
由于初赛 DC-ROMA 未申请成功，所以无法在 riscv64 硬件上测试运行，只能在 docker 中模拟 riscv64 环境。

## 使用方法

### 本地构建

```bash
docker build -t electron-builder-riscv .
```

然后进入容器

```bash
docker run -it --name electron-builder-riscv -v electron-builder-riscv:lastest
```

在容器中运行

```bash
cd my-electron-app
electron-builder --linux tar.gz
```

当然也可以将原有的项目文件挂载到容器中

```bash
docker run -it --name electron-builder-riscv -v /path/to/your/project:/home/my-electron-app electron-builder-riscv:lastest
```

但要注意 package.json 中的`build`字段需要修改,且原项目不要安装`electron-builder`工具,因为容器中已经安装了全局的`electron-builder`工具。

```json
"build": {
    "electronVersion": "30.0.0",
    "electronDist": "/opt/electron"
  }
```

### 使用已构建的镜像

```bash
docker pull sd0ric4/electron-builder-riscv:1.0
```

接下来的步骤和`本地构建`中的步骤一样。

## 项目简介

> Electron 是一个使用 JavaScript、HTML 和 CSS 构建跨平台的桌面应用程序，它基于 Node.js 和 Chromium。本项目的目标是让 Electron 项目能在 RISC-V 平台上进行打包和构建，使构建出来的应用程序能在 RISC-V 平台运行。

项目目前实现了在`docker`中的`riscv64`环境下的`arch`系统，使用`electron-builder`工具成功进行打包构建，生成 deb 的 tar.gz 文件。

## 项目过程

### 项目准备

之所以选择这个项目的原因是因为对于`electron`和`riscv`有着浓厚的兴趣。我一开始就是学前端的，所以已经对于`electron`有一定的了解。而`riscv`是一个新兴的架构，我也对它有着浓厚的兴趣。所以这个项目是我两个兴趣的结合。

因此第一步是广泛的搜索资料。

### electron on riscv

在这个时候我发现了一个关于在 `riscv` 架构上支持`electron`的 issue:

[[Feature Request]: Possible RISC-V Support #32551](https://github.com/electron/electron/issues/32551)

![the issue](assets/README/image-5.png)
该`issue`里我找到了一个第三方重新编译`electron`的项目

- [交叉编译 electron 的方法](https://github.com/riscv-forks/electron)
- [releases of elecrton on riscv](https://github.com/riscv-forks/electron-riscv-releases)

通过这个项目，我成功的在`riscv64`系统上安装了`electron`。
![alt text](assets/README/image.png)

```bash
cd /home
wget https://github.com/riscv-forks/electron-riscv-releases/releases/download/v30.0.7/electron-v30.0.7-linux-riscv64.zip
mkdir electron
unzip electron-v30.0.7-linux-riscv64.zip -d electron
mkdir /opt/electron
cp -r /home/electron/* /opt/electron
ln -s /opt/electron/electron /usr/bin/electron
```

使用 ⬆️ 上面的命令可以在`riscv64`系统上安装`electron`。

### archriscv

接下来我需要做的是熟悉`riscv`架构，很可惜我没有一台充足性能的`riscv`开发板。项目初期好像无法申请到那台题目要求的`riscv`笔记本。

已知`ubuntu`和`arch`有支持`riscv64`的版本，而我是`arch`用户，所以我选择了`arch`系统。

使用以下命令可以快速启动一个无图形界面的`riscv64`系统

```bash
# pacman -Syu then reboot is recommended before this
sudo pacman -S arch-install-scripts git qemu-img qemu-system-riscv riscv64-linux-gnu-gcc devtools-riscv64
git clone https://github.com/CoelacanthusHex/archriscv-scriptlet.git
cd archriscv-scriptlet
./mkrootfs
./mkimg
./startqemu.sh
```

然而上面的命令只能启动一个无图形界面的`riscv64`系统，而我需要一个有图形界面的`riscv64`系统。
![arch riscv](assets/README/image-6.png)
在使用的过程中学习到了很多关于`qemu`的知识，并且成功在`arch`系统上运行了`riscv64`的`arch`系统，使用 xfce 桌面环境。
![arch xfce1](assets/README/b30f243272bd1d56927e4374bed160f5_720.png)
![arch xfce2](assets/README/8a99329b0d39fccd6330a99120bea87f_720.png)
然而当时这种情况复现的难度较大，而且由于最近上游更新了 libicu，却还没更新`electron`的版本，由于 arch 是滚动系统，在更新 libicu 后`archriscv`的`electron`无法正常运行。

为了解决这个问题，我决定使用`docker`来模拟`riscv64`的环境。

![vscode 连接riscv失败](assets/README/image-1.png)

`electron-builder`本身并不支持`riscv64`平台
![alt text](assets/README/image-2.png)

![alt text](assets/README/image-3.png)

### electron-builder on riscv

要使`electron-builder`工具支持`riscv64`平台，需要重新编译`electron-builder`工具。

而通过搜索我找到了一个`electron-builder`的`fork`的`npm`包，这个项目已经支持了`riscv64`平台。

尽管找到了这个项目，仍然有一些问题，比如缺乏 7za 文件，但是本身就可以安装`riscv64`的`7za`，所以我们可以通过创建一个软链接来解决这个问题。
![alt text](assets/README/image-7.png)

```bash
mkdir -p /usr/lib/node_modules/@riscv-forks/electron-builder/node_modules/7zip-bin/linux/riscv64
ln -s /usr/bin/7za /usr/lib/node_modules/@riscv-forks/electron-builder/node_modules/7zip-bin/linux/riscv64/7za
```

#### app-builder on riscv

同样的问题也出现在`app-builder`和`builder-util`上，我们可以通过同样的方法解决这个问题。
![alt text](assets/README/image-8.png)
但是我发现`app-builder`本身已经支持了`riscv64`平台，所以我们可以直接使用`app-builder`工具。

```bash
mkdir /home/tmp
cd /home/tmp
wget https://github.com/develar/app-builder/archive/refs/tags/v4.2.0.zip
mkdir app-builder
unzip v4.2.0.zip -d app-builder
cd app-builder/app-builder-4.2.0/
go build
mkdir -p /usr/lib/node_modules/@riscv-forks/electron-builder/node_modules/app-builder-bin/linux/riscv64
ln -s /home/tmp/app-builder/app-builder-4.2.0/app-builder /usr/lib/node_modules/@riscv-forks/electron-builder/node_modules/app-builder-bin/linux/riscv64/app-builder
```

### 成功构建

![alt text](assets/README/image-4.png)

## 开源协议

本项目遵循大赛要求，比赛代码，技术文档，答辩材料均开源。

### 源代码开源协议

项目源代码遵循 MIT 开源协议。

### 文档开源协议

项目文档遵循 CC-BY-SA 4.0 开源协议。

## 初赛总结

初赛时我未能申请到题目要求的 riscv 笔记本，因此无法在 riscv64 硬件上测试运行，只能在 Docker 中模拟 riscv64 环境。然而，我成功在 riscv64 环境下的 Arch 系统中使用 electron-builder 工具进行打包构建，生成了 deb 的 tar.gz 文件。在这个过程中，我学到了很多，熟练了 Docker 和 QEMU 的使用，对 RISC-V 和 Electron 有了更深入的了解。感谢这次比赛，让我收获颇丰。


